API_NOT_IMPLEMENTED = "err_api_not_implemented"
API_VERSION_NOT_FOUND = "err_api_version_not_found"
API_INTERNAL_ERROR = "err_api_internal_error"
API_MISSING_REQUEST_ID = "err_api_missing_request_id"
EXPIRED_AUTHORIZATION_TOKEN = "err_authorization_token_expired"
USER_BLOCKED = "err_user_blocked"
API_RESOURCE_NOT_FOUND = "err_api_resource_not_found"
SERVICE_NOT_FOUND = "err_api_service_not_found"
API_RESOURCE_ALREADY_EXISTS = "err_api_resource_already_exists"
API_INVALID_ENTITY_STATE = "err_api_invalid_entity_state"
INVALID_DATA = "err_api_invalid_data"
API_UNPARSABLE_JSON = "err_unparseable_input"
INVALID_AUTHENTICATION_CREDENTIALS = "err_auth_invalid_credentials"
UNHANDLED_EXCEPTION = "err_unknown_exception"
INVALID_AUTHENTICATION_STATE = "err_auth_invalid_state"
ERR_ACCESS_DENIED = "err_acl_access_denied"
EMPTY_QUEUE = "wrn_nothing_queued_for_execution"
API_STORAGE_NONE_EMPTY_FOLDER = "err_storage_folder_none_empty"
API_SERVICE_DOWN_ERROR = "err_external_service_down"
API_METHOD_NOT_ALLOWED = "err_method_not_allowed"
