VERSION = '1.0.7'
VERSION_HASH = 'cc653354eb327c37adc66dfd11163ea0c919e00c'
BUILD_DATE = '2024-01-29 12:28:51 UTC'

HEADER_REQUEST_ID = "X-CONVERGENCE-REQUEST-ID"
HEADER_CALLER_SERVICE = "X-CONVERGENCE-CALLER-SERVICE"
HEADER_CALLER_SERVICE_HASH = "X-CONVERGENCE-CALLER-SERVICE-HASH"
HEADER_CALLER_SERVICE_VERSION = "X-CONVERGENCE-CALLER-SERVICE-VERSION"
HEADER_PARENT_REQUEST_ID = "X-CONVERGENCE-PARENT-REQUEST-ID"
